import matplotlib.pyplot as plt
import matplotlib.collections
import matplotlib.patches
import numpy as np
from matplotlib import rc


rc('text', usetex=True)
rc('text.latex', preamble="\n".join([
    '\\usepackage{amsmath}',
    '\\newcommand{\\D}{\mathrm{d}}'
]))


def errorband(p, ax=None, underflow=False, overflow=False):
    if ax is None:
        ax = plt.gca()

    p = p.copy()

    delta = p[2,0] - p[1,0]

    if p[0,0] == -np.inf:
        if underflow is False:
            p = p[1:]
        else:
            s = 2 if type(underflow) == bool else underflow
            p[0,0] = p[1,0] - (s+0.5) * delta
    if p[-1,0] == +np.inf:
        if overflow is False:
            p = p[:-1]
        else:
            s = 2 if type(overflow) == bool else overflow
            p[-1,0] = p[-2,0] + (s+0.5) * delta

    boxes = [
        matplotlib.patches.Rectangle(
            (x - delta, y - e/2), delta, e
        )
        for x, y, e in p
    ]

    artist = ax.step(p[:,0], p[:,1])
    col = artist[0].get_color()

    pc = matplotlib.collections.PatchCollection(
        boxes,
        facecolor=col, edgecolor=col
    )

    ax.add_collection(pc)
    return artist


def twopanel(labelx,
             upleft=[], labupleft="",
             downleft=[], labdownleft="",
             upright=[], labupright="",
             downright=[], labdownright=""):

    if type(upleft) == np.ndarray:
        upleft = [upleft]
    if type(upright) == np.ndarray:
        upright = [upright]
    if type(downleft) == np.ndarray:
        downleft = [downleft]
    if type(downright) == np.ndarray:
        downright = [downright]

    fig, axs = plt.subplots(
        2, sharex=True, sharey=True, gridspec_kw={'hspace': 0}
    )
    axs[1].set_xlabel(labelx)

    axs[0].set_ylabel(labupleft)
    for i in upleft:
        errorband(i, ax=axs[0])

    if len(upright) > 0:
        ax2 = axs[0].twinx()
        if labupright is not None:
            ax2.set_ylabel(labupright)
        for i in upright:
            errorband(i, ax=ax2)

    axs[1].set_ylabel(labdownleft)
    for i in downleft:
        errorband(i, ax=axs[1])

    if len(downright) > 0:
        ax3 = axs[1].twinx()
        if labdownright is not None:
            ax3.set_ylabel(labdownright)
        for i, c in downright:
            errorband(i, ax=ax3)

    return fig, axs
